#!/bin/bash
#  $HeadURL: https://svn.pasta.freemyip.com/main/bs/tags/2/share/steps/test-pkgctrl.sh $ $LastChangedRevision: 10922 $

#  Modules
. $(bs-config bs_share_prefix)/include/bssupport.sh || { echo "${0##*/}: INTERNAL ERROR: failed to load bssupport.sh" >&2; exit 3; }
. $(miniade) || { echo "${0##*/}: ERROR: miniade failed (hint: run 'miniade' to see error)" >&2; exit 1; }

#  Configurable stuff

#  Other globals

main()
{
    local MY_ARGS
    local PROGRAM VERBOSELEVEL PROGNAME AGGREGATED_RC

    #  Defaults for options

    #  Process options
    miniade_process_options --help-handler=usage MY_ARGS "$@" && set -- "${MY_ARGS[@]}"

    #  Process arguments

    #  Sanity checks and derivations
    #  bs sets PROGRAM, but passes it to us through environment as BS_PROGRAM, copy it back.
    PROGRAM=$BS_PROGRAM   
    VERBOSELEVEL=$BS_VERBOSELEVEL   
    PKGCTRL_SVNWC_DIR=$BS_PKGCTRL_SVNWC_DIR   
    PKGCTRL_SVNREPO_URL=$BS_PKGCTRL_SVNREPO_URL   
    DELAY_FAIL_ENABLED_FLAG=$BS_DELAY_FAIL_ENABLED_FLAG
    bssupport_checkset VERBOSELEVEL PROGRAM PKGCTRL_SVNWC_DIR PKGCTRL_SVNREPO_URL
    #  DEBEMAIL and DEBFULLNAME must be set in the developer's environment. They
    #  are not bs-specific: they are used by dch and other deb-related commands.
    bssupport_checkset DEBEMAIL DEBFULLNAME
    miniade_get_progname PROGNAME
    miniade_set_verboselevel $VERBOSELEVEL
    bssupport_changestate --id=$PROGRAM.debian --from=unreleased --step=$PROGNAME
    bssupport_sandbox
    
    #  Guts
    #  Recently, I switched to *not* checking test output for errors, but only
    #  checking return codes. That means that *any* error detected in the checks
    #  below needs to result in a non-zero exit code. However, when some checks
    #  fail it is nonetheless useful to run a few more checks before exiting due
    #  to the check that failed. So we split these checks into phases.
    
    #  Start phase #1
    RC=0
    AGGREGATED_RC=$RC
    
    miniade_info "phase #1.1: getting pkgtrl ..."
    cp -r $PKGCTRL_SVNWC_DIR $PROGRAM.debian || RC=$?
    
    miniade_info "phase #1.2: checking is unreleased ..."
    #  This *should* be redundant as the changelog should be in step with bs's 
    #  state management. But actually we need to *know* the version in order
    #  to check if a newer version is available (see below). Note that
    #  get_unreleased_pkgctrl_version() calls error itself, if it fails,
    #  but we'll treat it as if it didn't do that, just for symmetry.
    bssupport_get_unreleased_pkgctrl_version $PROGRAM.debian/changelog CHANGELOG_VERSION || RC=$?
    
    #  End phase #1
    if [ $RC = 0 ]; then
        :
    elif $DELAY_FAIL_ENABLED_FLAG; then
        miniade_warning "something in phase #1 failed (hint: see messages above); exit delayed"
        AGGREGATED_RC=$RC
    else
        miniade_error "something in phase #1 failed (hint: see messages above)"
    fi
    
    #  Start phase #2
    RC=0
    
    miniade_info "phase #2.1: checking for newer source release ..."
    #  It would be nice to move into the 'Miscellaneous other checks' stanza
    #  below, but it's really a total waste of time to do any of the later
    #  checks (e.g. pullable, pushable) before checking that the package control
    #  files reference the latest source release.
    AVAILABLE_VERSION=$(ln -s $PROGRAM.debian debian; uscan --noconf --no-download --verbose 2>/dev/null | sed -n 's/uscan: Newest version of .* on remote site is \([^,]*\),.*/\1/p'; rm debian)
    miniade_debug 10 "AVAILABLE_VERSION=$AVAILABLE_VERSION, CHANGELOG_VERSION=$CHANGELOG_VERSION"
    [ "X$AVAILABLE_VERSION" = X ] || dpkg --compare-versions $CHANGELOG_VERSION eq $AVAILABLE_VERSION || { miniade_warning "source release version $AVAILABLE_VERSION is available but changelog is for version $CHANGELOG_VERSION!"; RC=1; }
    
    miniade_info "phase #2.2: checking for pullable stuff ..."
    SVN_OUTPUT=$(svn st -u $PKGCTRL_SVNWC_DIR | grep '\*' 2>&1) || true
    [ "X$SVN_OUTPUT" = X ] || { echo "$SVN_OUTPUT"; RC=1; }
    
    miniade_info "phase #2.3: checking for pushable stuff ..."
    SVN_OUTPUT=$(svn st $PKGCTRL_SVNWC_DIR | grep -v '^\?' 2>&1) || true
    [ "X$SVN_OUTPUT" = X ] || { echo "$SVN_OUTPUT"; RC=1; }
    
    miniade_info "phase #2.4: checking for untagged stuff ..."            
    CHECK_OUTPUT=$(find $PKGCTRL_SVNWC_DIR -name '.svn' -prune -o -type f -print | while read FILE; do
        ! expr "$FILE" : '.*/\(compat\|templates\|copyright\|control\|changelog\|source/format\)$' > /dev/null || continue
        [ "X$(svn pget svn:keywords $FILE 2>/dev/null | paste -d' ' -s)" = "XHeadURL LastChangedRevision" ] || echo "$FILE: no keywords property"
        #  Grep for the keywords but write the keywords in such a way that *this script* won't
        #  have these keywords changed by svn!
        grep -q '\$Hea[d]URL: .*\$ \$LastCha[n]gedRevision: .* \$' $FILE || echo "$FILE: no keyword placeholders"
    done)
    [ "X$CHECK_OUTPUT" = X ] || { echo "$CHECK_OUTPUT"; RC=$?; }
    
    #  End phase #2
    if [ $RC = 0 ]; then
        :
    elif $DELAY_FAIL_ENABLED_FLAG; then
        miniade_warning "something in phase #2 failed (hint: see messages above); exit delayed"
        AGGREGATED_RC=$RC
    else
        miniade_error "something in phase #2 failed (hint: see messages above)"
    fi
    
    #  Start phase #3
    RC=0
    
    miniade_info "phase #3.1: checking packagable ..."
    #  Order is important here
    PARENT_SANDBOX=$SANDBOX SANDBOX= $BSSUPPORT_STEP_MAKE_PACKAGE_CMD || RC=$?
    
    miniade_info "phase #3.2: checking for unignored stuff ..."
    SVN_OUTPUT=$(svn st $PKGCTRL_SVNWC_DIR | grep '^\?' 2>&1) || true
    [ "X$SVN_OUTPUT" = X ] || { echo "$SVN_OUTPUT"; RC=1; }
    
    #  End phase #3
    if [ $RC = 0 ]; then
        :
    elif $DELAY_FAIL_ENABLED_FLAG; then
        miniade_warning "something in phase #3 failed (hint: see messages above); exit delayed"
        AGGREGATED_RC=$RC
    else
        miniade_error "something in phase #3 failed (hint: see messages above)"
    fi
    
    #  Start phase #4
    RC=0
    
    DEB=$(echo $SANDBOX/*.deb)
    miniade_info "phase #4.1: checking lintian ..."
    #   Debian 10 said 'X hint[s] overridden' but Debian 11 says 'X tag[s] overridden'. Allow both.
    LINTIAN_OUTPUT=$(lintian $DEB 2>&1 | egrep -v "^N: .* (hint|tag)s? overridden .*") || true
    [ "X$LINTIAN_OUTPUT" = X ] || { echo "$LINTIAN_OUTPUT"; RC=1; }
    
    miniade_info "phase #4.2: checking no 'HeadURL' in conf files ..."
    CONF_FILES=$(dpkg-deb -I $DEB conffiles 2>/dev/null | sed 's/^/./')
    if [ "X$CONF_FILES" != X ]; then
        GREP_OUTPUT=$(ar p $DEB data.tar.xz | tar xJfO - $CONF_FILES | grep -i headurl) || true
        [ "X$GREP_OUTPUT" = X ] || { echo "$GREP_OUTPUT"; RC=1; }
    fi
    
    #  End phase #4
    if [ $RC = 0 ]; then
        :
    elif $DELAY_FAIL_ENABLED_FLAG; then
        miniade_warning "something in phase #3 failed (hint: see messages above); exit delayed"
        AGGREGATED_RC=$RC
    else
        miniade_error "something in phase #3 failed (hint: see messages above)"
    fi

    [ $AGGREGATED_RC = 0 ] || miniade_error "exiting after delayed exit"
}

main "$@"
